#ifndef UOSLCD_H
#define UOSLCD_H
/*************************************************************************
Ansteuer-Routinen fr die LCD-Anzeige
20061029  KB  UniOs
4-Bit-Interface ber Port D (Data-Nibble auf oberen 4 Bits)
LCD_Init() initialisisert die LCD-Anzeige
LCD_Line(Line, String) gibt einen String auf Line (0 oder 1) aus
LCD_Clear() lscht die Anzeige
*************************************************************************/

#define XTAL        8       // CPU-Frequenz
#define LCDPort     PORTD     // Ausgabeport D
#define LCDDDR      DDRD
#define PIN_Enable  _BV(0)    // Enable = Bit 0 
#define PIN_RS      _BV(1)    // RS = Bit 1
#define DATAMASK    0xf0      // 4-Bit-Nibble auf DB4-DB7

#include <avr/io.h>
#include "uosdelay.h"

//erzeugt den Enable-Puls
void LCD_Enable()
{
  LCDPort |= PIN_Enable;
  Delay_us(2);
  LCDPort &= ~PIN_Enable;
}

// Ausgabe eines 4-Bit_Nibbles
static inline void LCD_Nibble(uint8_t N)
{
  LCDPort = N;
  LCD_Enable();
}

void LCD_Data(uint8_t b)
{
  LCD_Nibble((b&DATAMASK) |  PIN_RS);
  LCD_Nibble((b<<4) |  PIN_RS);
  Delay_us(100);
}

void LCD_Command(uint8_t b)
{
  LCD_Nibble(b&DATAMASK);
  LCD_Nibble(b<<4);
  Delay_us(100);
}

static inline void LCD_Clear()
{
  LCD_Command(0b00000001);
  Delay_ms(3);
}

static inline void LCD_Address(uint8_t addr)
{
  LCD_Command(0x80 | ((addr) & 0x7f));
}

void LCD_Init()
{
  LCDDDR = 0xff;           //Port als Ausgang programmieren
  Delay_ms(200);
  LCD_Nibble(0b00110000);  //8bit-Grundmodus einstellen
  Delay_ms(5);
  LCD_Enable();            //3 Enable-Pulse !?!?        
  Delay_ms(5);
  LCD_Enable();
  Delay_ms(5);
  LCD_Nibble(0b00100000);  //4bit-Modus einstellen
  Delay_ms(5);
  LCD_Command(0b00101000); //2-zeilig
  LCD_Command(0b00001100); //Display=on Cursor=off blink=off
  LCD_Clear();
}

void LCD_Line(int line, char * buffer)
{
  if (line==0)
    LCD_Address(0);
  else
    LCD_Address(40);
  while (*buffer)
  {
    LCD_Data(*buffer);
    buffer++;
  }
}
#endif // UOSLCD_H
